@@ -1,7 +1,7 @@
 package Perinci::CmdLine::Base;
 
-our $DATE = '2015-02-27'; # DATE
-our $VERSION = '0.88'; # VERSION
+our $DATE = '2015-03-05'; # DATE
+our $VERSION = '0.89'; # VERSION
 
 use 5.010001;
 use strict;
@@ -442,12 +442,6 @@ sub do_completion {
 
     shift @$words; $cword--; # strip program name
 
-    {
-        my $env_words = $self->_read_env($r);
-        unshift @$words, @$env_words;
-        $cword += @$env_words;
-    }
-
     # @ARGV given by bash is messed up / different. during completion, we
     # get ARGV from parsing COMP_LINE/COMP_POINT.
     @ARGV = @$words;
@@ -456,6 +450,12 @@ sub do_completion {
     # command-line argument, or default_subcommand.
     $self->_parse_argv1($r);
 
+    {
+        my $env_words = $self->_read_env($r);
+        unshift @ARGV, @$env_words;
+        $cword += @$env_words;
+    }
+
     #$log->tracef("ARGV=%s", \@ARGV);
     #$log->tracef("words=%s", $words);
 
@@ -637,6 +637,11 @@ sub _parse_argv2 {
 
     my %args;
 
+    {
+        my $env_words = $self->_read_env($r);
+        unshift @ARGV, @$env_words;
+    }
+
     # parse argv for per-subcommand command-line opts
     if ($r->{skip_parse_subcommand_argv}) {
         return [200, "OK (subcommand options parsing skipped)"];
@@ -1044,11 +1049,6 @@ sub run {
         $log->tracef("[pericmd] Running hook_before_run ...");
         $self->hook_before_run($r);
 
-        {
-            my $env_words = $self->_read_env($r);
-            unshift @ARGV, @$env_words;
-        }
-
         my $parse_res = $self->parse_argv($r);
         if ($parse_res->[0] == 501) {
             # we'll need to send ARGV to the server, because it's impossible to
@@ -1142,7 +1142,7 @@ Perinci::CmdLine::Base - Base class for Perinci::CmdLine{Classic,::Lite}
 
 =head1 VERSION
 
-This document describes version 0.88 of Perinci::CmdLine::Base (from Perl distribution Perinci-CmdLine-Lite), released on 2015-02-27.
+This document describes version 0.89 of Perinci::CmdLine::Base (from Perl distribution Perinci-CmdLine-Lite), released on 2015-03-05.
 
 =head1 DESCRIPTION
 
